// Adafruit Motor shield library
// copyright Adafruit Industries LLC, 2009
// this code is public domain, enjoy!



#if defined(__AVR__)
	#include <avr/io.h>
#endif


#include "AFMotor.h"

/*
If you are using all of the motors, it does take up most of the digital IOs, but ONLY if you're using those motors:
All 6 analog input pins are available. They can also be used as digital pins (pins #14 thru 19)
Digital pin 2, and 13 are not used.
The following pins are in use only if the DC/Stepper noted is in use:
Digital pin 11: DC Motor #1 / Stepper #1 (activation/speed control)
Digital pin 3: DC Motor #2 / Stepper #1 (activation/speed control)
Digital pin 5: DC Motor #3 / Stepper #2 (activation/speed control)
Digital pin 6: DC Motor #4 / Stepper #2 (activation/speed control)

The following pins are in use if any DC/steppers are used
Digital pin 4, 7, 8 and 12 are used to drive the DC/Stepper motors via the 74HC595 serial-to-parallel latch

http://www.jaxcoder.com/Products.aspx?id=7
*/

static uint8_t latch_state;

AFMotorController::AFMotorController(void) {
	TimerInitalized = false;
}

void AFMotorController::enable(void) {
	// setup the latch

	LATCH_DDR |= (1<<LATCH);
	ENABLE_DDR |= (1<<ENABLE);
	CLK_DDR |= (1<<CLK);
	SER_DDR |= (1<<SER);

	latch_state = 0;

	latch_tx();  // "reset"

	ENABLE_PORT &= ~(1<<ENABLE); // enable the chip outputs!
}


void AFMotorController::latch_tx(void) {
	uint8_t i;

	LATCH_PORT &= ~(1<<LATCH);

	SER_PORT &= ~(1<<SER);

	for (i=0; i<8; i++) {
		CLK_PORT &= ~(1<<CLK);

		if (latch_state & (1<<(7-i))) {
			SER_PORT |= (1<<SER);
		} else {
			SER_PORT &= ~(1<<SER);
		}
		CLK_PORT |= (1<<CLK);
	}
	LATCH_PORT |= (1<<LATCH);
}

static AFMotorController MC;

/******************************************
               MOTORS
******************************************/
inline void initPWM1(uint8_t freq) {
	// use PWM from timer2A on PB3 (Arduino pin #11) 
	// 324PA port PD7
	TCCR2A |= (1<<COM2A1) | (1<<WGM20) | (1<<WGM21); // fast PWM, turn on oc2a
	TCCR2B = freq & 0x7;
	OCR2A = 0;
}

inline void setPWM1(uint8_t s) {
	// use PWM from timer2A on PB3 (Arduino pin #11)
	OCR2A = s;
}

inline void initPWM2(uint8_t freq) {
	// use PWM from timer2B (pin 3) PD3
	// 324PA port PD6
	TCCR2A |= (1<<COM2B1) | (1<<WGM20) | (1<<WGM21); // fast PWM, turn on oc2b
	TCCR2B = freq & 0x7;
	OCR2B = 0;
}

inline void setPWM2(uint8_t s) {
	// use PWM from timer2A on PB3 (Arduino pin #11)
	OCR2B = s;
}

inline void initPWM3(uint8_t freq) {
	// use PWM from timer0A / PD6 (pin 6)
	// 324 PA port PB3
	TCCR0A |= (1<<COM0A1) | (1<<WGM00) | (1<<WGM01); // fast PWM, turn on OC0A
	TCCR0B = freq & 0x7;
	OCR0A = 0;
}

inline void setPWM3(uint8_t s) {
	// use PWM from timer0A on PB3 (Arduino pin #6)
	OCR0A = s;
}

inline void initPWM4(uint8_t freq) {
	// use PWM from timer0B / PD5 (pin 5)
	// 324 PA port PB4
	TCCR0A |= (1<<COM0B1) | (1<<WGM00) | (1<<WGM01); // fast PWM, turn on oc0a
	//TCCR0B = freq & 0x7;
	OCR0B = 0;
}

inline void setPWM4(uint8_t s) {
	// use PWM from timer0A on PB3 (Arduino pin #6)
	OCR0B = s;
}

AF_DCMotor::AF_DCMotor(uint8_t num, uint8_t freq) {
	motornum = num;
	pwmfreq = freq;

	MC.enable();

	switch (num) {
		case 1:
			latch_state &= ~(1<<MOTOR1_A) & ~(1<<MOTOR1_B); // set both motor pins to 0
			MC.latch_tx();
			initPWM1(freq);
			break;
		case 2:
			latch_state &= ~(1<<MOTOR2_A) & ~(1<<MOTOR2_B); // set both motor pins to 0
			MC.latch_tx();
			initPWM2(freq);
			break;
		case 3:
			latch_state &= ~(1<<MOTOR3_A) & ~(1<<MOTOR3_B); // set both motor pins to 0
			MC.latch_tx();
			initPWM3(freq);
			break;
		case 4:
			latch_state &= ~(1<<MOTOR4_A) & ~(1<<MOTOR4_B); // set both motor pins to 0
			MC.latch_tx();
			initPWM4(freq);
			break;
	}
}

void AF_DCMotor::run(uint8_t cmd) {
	uint8_t a, b;
	switch (motornum) {
		case 1:
			a = MOTOR1_A; 
			b = MOTOR1_B; 
			break;
		case 2:
			a = MOTOR2_A; 
			b = MOTOR2_B; 
			break;
		case 3:
			a = MOTOR3_A; 
			b = MOTOR3_B; 
			break;
		case 4:
			a = MOTOR4_A; 
			b = MOTOR4_B; 
			break;
		default:
			return;
	}

	switch (cmd) {
		case FORWARD:
			latch_state |= (1<<a);
			latch_state &= ~(1<<b); 
			MC.latch_tx();
			break;
		case BACKWARD:
			latch_state &= ~(1<<a);
			latch_state |= (1<<b); 
			MC.latch_tx();
			break;
		case RELEASE:
			latch_state &= ~(1<<a);     // A and B both low
			latch_state &= ~(1<<b); 
			MC.latch_tx();
			break;
	}
}

void AF_DCMotor::setSpeed(uint8_t speed) {
	switch (motornum) {
		case 1:
			setPWM1(speed); 
			break;
		case 2:
			setPWM2(speed); 
			break;
		case 3:
			setPWM3(speed); 
			break;
		case 4:
			setPWM4(speed); 
			break;
	}
}


